// Col2Asp
// Time-stamp: <2023-05-03 09:48:32 irumisugimori>
// @author: Irumi Sugimori (irumi.sugimori@nagoya-u.jp)

import scala.io.Source
import scala.util.matching.Regex
import java.io.File

object Col2Asp {
  var isVerbose = false

  def main(args: Array[String]): Unit = {
    val optionPattern = "^-.*".r
    val options = args.filter(optionPattern.matches(_))

    // Check option
    for (opt <- options) {
      opt match {
        case "-h" =>
          usage()
        case "--help" =>
          usage()
        case "-verbose" =>
          isVerbose = true
        case _ =>
          printError(s"'$opt' is wrong option")
          usage()
      }
    }

    val filenames = args.filterNot(optionPattern.matches(_))

    // Check number of arguments
    if (filenames.size < 1) {
      printError("Filename is not given")
      usage()
    }

    val filename = filenames(0)

    // Check existence of file
    if (!new File(filename).exists()) {
      printError(s"File '$filename' is not found")
      usage()
    }

    val file = Source.fromFile(filename)
    val programPattern = "^p edge ([0-9]+) ([0-9]+)$".r
    val edgePattern = "^e ([0-9]+) ([0-9]+)$".r
    val lengthPattern = "^l ([0-9]+)$".r
    val terminalPattern = "^t ([0-9]+) ([0-9]+)$".r
    val commentPattern = "^c .*$|^c$".r

    // Conversion
    try {
      for (line <- file.getLines) {
        printComment(line)
        line.strip match {
          case programPattern(n,m) =>
            printFact("num_of_nodes",List(n))
            printFact("num_of_edges",List(m))
            printFact("node",List(s"1..$n"))
          case edgePattern(v1,v2) =>
            printFact("edge",List(v1,v2))
          case lengthPattern(len) =>
            printFact("max_length",List(len))
          case terminalPattern(v1,v2) =>
            printFact("terminal",List(v1,v2))
          case commentPattern() =>
          case "" =>
          case _ =>
            throw new RuntimeException(s"'$line' is wrong format")
        }
      }
    } catch {
      case e: Exception => printError(e.getMessage)
    } finally {
      file.close
    }
  }

  def usage(): Unit = {
    printMsg("Usage:", "scala Col2Asp <file> [options]")
    printMsg("","<file>         filename of COL file")
    printMsg("","-h, --help     print this usage text")
    printMsg("","-v, --verbose  output verbose comments")
    sys.exit(1)
  }

  def printError(msg: String): Unit = {
    printMsg("Error:", msg)
  }

  def printComment(comment: String): Unit = {
    if (isVerbose) {
      printMsg("%", comment)
    }
  }

  def printMsg(head: String, body: String): Unit = {
    println(s"$head $body")
  }

  def printFact(predicate: String, argsList: List[String]): Unit = {
    val args = argsList.mkString(",")
    println(s"$predicate($args).")
  }
}
